#
# spec file for package spark
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
# Copyright (c) 2017 cadenzajon
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%define sname spark
%define username spark
%define groupname spark
%define sparkdir %{_datadir}/%{sname}
%define workername %{sname}-worker
%define mastername %{sname}-master
%define scalaversion 2.10
Name:           %{sname}
Version:        2.2.3
Release:        3.1
Summary:        Unified Analytics Engine for Big Data
License:        Apache-2.0
Group:          Development/Libraries/Java
Url:            http://spark.apache.org/
Source0:        http://apache.org/dist/spark/spark-%{version}/spark-%{version}.tgz
Source1:        build.sh
Source2:        %{workername}.service
Source3:        %{mastername}.service
Source7:        spark-tmpfiles.conf
BuildRequires:  fdupes
BuildRequires:  java-1_8_0-openjdk
BuildRequires:  java-1_8_0-openjdk-devel
BuildRequires:  spark-kit = %{version}
BuildRequires:  systemd-rpm-macros
BuildRequires:  tar
BuildRequires:  xz
%{?systemd_requires}
Requires:       java-1_8_0-openjdk
Requires(pre):  shadow
BuildArch:      noarch

%description
Apache Spark is a fast and general-purpose cluster computing system. It provides
high-level APIs in Java, Scala, Python and R, and an optimized engine that
supports general execution graphs. It also supports a rich set of higher-level
tools including Spark SQL for SQL and structured data processing, MLlib for
machine learning, GraphX for graph processing, and Spark Streaming.

%prep
%setup -q -c -n src
cp -f %{SOURCE1} .
cp -Rf %{_datadir}/tetra ../kit

%build
cd ..
VERSION=%{version} sh src/build.sh

%install
export NO_BRP_CHECK_BYTECODE_VERSION=true

# directories
install -d -m 755 %{buildroot}%{_bindir}
install -d -m 755 %{buildroot}%{_sbindir}
install -d -m 755 %{buildroot}%{_unitdir}
install -d -m 755 %{buildroot}/%{_sysconfdir}/%{name}
install -d -m 750 %{buildroot}%{_localstatedir}/log/%{name}
install -d -m 755 %{buildroot}%{_localstatedir}/lib/%{name}
install -d -m 755 %{buildroot}%{sparkdir}

cp -a %{name}-%{version}/dist/* %{buildroot}%{sparkdir}

# jars, note other versions of spark-streaming-kafka are available from the kit
install -D -m 644 %{name}-%{version}/./external/kafka-0-8/target/spark-streaming-kafka-0-8_%{scalaversion}-%{version}.jar %{buildroot}%{sparkdir}/jars/spark-streaming-kafka-0-8_%{scalaversion}-%{version}.jar

# systemd
install -D -m 444 %{SOURCE2} %{buildroot}%{_unitdir}/%{workername}.service
install -D -m 444 %{SOURCE3} %{buildroot}%{_unitdir}/%{mastername}.service
ln -s %{_sbindir}/service %{buildroot}%{_sbindir}/rc%{workername}
ln -s %{_sbindir}/service %{buildroot}%{_sbindir}/rc%{mastername}

# tmpfiles
install -D -m 644 %{SOURCE7} %{buildroot}/%_tmpfilesdir/spark.conf

# bin files
ln -sr %{buildroot}/%{sparkdir}/bin/spark-submit %{buildroot}%{_bindir}/spark-submit
ln -sr %{buildroot}/%{sparkdir}/bin/spark-class %{buildroot}%{_bindir}/spark-class
ln -sr %{buildroot}/%{sparkdir}/bin/spark-shell %{buildroot}%{_bindir}/spark-shell
ln -sr %{buildroot}/%{sparkdir}/bin/spark-sql %{buildroot}%{_bindir}/spark-sql
ln -sr %{buildroot}/%{sparkdir}/bin/find-spark-home %{buildroot}%{_bindir}/find-spark-home

# set SPARK_HOME and SPARK_CONF_DIR, so that spark CLI tools can find spark libraries and configuration files
echo 'export SPARK_HOME=%{_datadir}/spark' > %{buildroot}%{sparkdir}/bin/find-spark-home
echo 'export SPARK_CONF_DIR=%{_sysconfdir}/spark' >> %{buildroot}%{sparkdir}/bin/find-spark-home

# config files
install -D -m 644 %{name}-%{version}/dist/conf/spark-defaults.conf.template %{buildroot}/%{_sysconfdir}/spark/spark-defaults.conf
install -D -m 755 %{name}-%{version}/dist/conf/spark-env.sh.template %{buildroot}/%{_sysconfdir}/spark/spark-env
# adjust default config (spark-env.sh is read during startup)
cat <<EOF >> %{buildroot}/%{_sysconfdir}/spark/spark-env

SPARK_CONF_DIR=/etc/spark/
SPARK_LOG_DIR=/var/log/spark/
SPARK_PID_DIR=/run/spark/
SPARK_CLASSPATH=/usr/share/spark/lib/*
SPARK_WORKER_DIR=/run/spark/work/
SPARK_MASTER_HOST=127.0.0.1
# The spark master addresses used by the workers
SPARK_MASTERS="spark://127.0.0.1:7077"
# The version of Scala used in the build
SPARK_SCALA_VERSION=2.10
EOF

rm -f %{sparkdir}/python/.gitignore
rm -rf %{sparkdir}/conf

%fdupes %{buildroot}%{sparkdir}

%pre
%service_add_pre %{workername}.service
%service_add_pre %{mastername}.service
# create system user/group and homedir
getent group %{groupname} >/dev/null || groupadd -r %{groupname}
getent passwd %{username} >/dev/null || useradd -r -g %{groupname} -d /var/lib/spark -s /sbin/nologin -c "user for Apache Spark" %{username}
exit 0

%post
%tmpfiles_create %{_tmpfilesdir}/spark.conf
%service_add_post %{workername}.service
%service_add_post %{mastername}.service

%preun
%service_del_preun %{workername}.service
%service_del_preun %{mastername}.service

%postun
%service_del_postun %{workername}.service
%service_del_postun %{mastername}.service

%files
%defattr(-,root,root)
%{sparkdir}
%{_unitdir}/spark-master.service
%{_unitdir}/spark-worker.service
%_tmpfilesdir/spark.conf
%{_sbindir}/rc%{workername}
%{_sbindir}/rc%{mastername}
%{_bindir}/spark-submit
%{_bindir}/spark-class
%{_bindir}/spark-shell
%{_bindir}/spark-sql
%{_bindir}/find-spark-home
%dir %attr(-,%{username},%{groupname}) /%{_sysconfdir}/%{name}
%dir %attr(0750, %{username}, %{groupname}) %{_localstatedir}/log/%{name}
%dir %attr(0755, %{username}, %{groupname}) %{_localstatedir}/lib/%{name}
%config(noreplace) %attr(-,%{username},%{groupname}) /%{_sysconfdir}/%{name}/spark-env
%config(noreplace) %attr(-,%{username},%{groupname}) /%{_sysconfdir}/%{name}/spark-defaults.conf

%changelog
* Tue Mar 12 2019 jodavis@suse.com
- Add metrics-core-2.2.0.jar and kafka_2.10-0.8.2.1.jar to dist/jars
- Note these version changes must match up with the spark-kit
  package, and that upstream OpenStack Monasca Transform is using
  Scala 2.10 and Kafka 0.8.
* Mon Mar 11 2019 jodavis@suse.com
- Changed spark-streaming-kafka to 0.8-2.10
  and changed copied file name to include versions
* Thu Mar  7 2019 jodavis@suse.com
- Changed Scala version in .jar filename to 2.10 to match build.sh
* Thu Mar  7 2019 johannes.grassler@suse.com
- Modified build.sh to build against Scala 2.10
* Mon Feb 18 2019 johannes.grassler@suse.com
- Build with -Phive and -Phive-thriftserver
- Replace upstream fix-spark-home by simplified one of our own
- Fix path for jars in service files
* Fri Feb 15 2019 johannes.grassler@suse.com
- Update to version 2.2.3
  * [SPARK-26327] - Metrics in FileSourceScanExec not update correctly while
    relation.partitionSchema is set
  * [SPARK-21402] - Fix java array of structs deserialization
  * [SPARK-22951] - count() after dropDuplicates() on emptyDataFrame returns
    incorrect value
  * [SPARK-23207] - Shuffle+Repartition on an DataFrame could lead to incorrect
  * answers
  * [SPARK-23243] - Shuffle+Repartition on an RDD could lead to incorrect
    answers
  * [SPARK-24603] - Typo in comments
  * [SPARK-24677] - TaskSetManager not updating successfulTaskDurations for old
    stage attempts
  * [SPARK-24809] - Serializing LongHashedRelation in executor may result in
    data error
  * [SPARK-24813] - HiveExternalCatalogVersionsSuite still flaky; fall back to
    Apache archive
  * [SPARK-24927] - The hadoop-provided profile doesn't play well with
    Snappy-compressed Parquet files
  * [SPARK-24948] - SHS filters wrongly some applications due to permission
    check
  * [SPARK-24950] - scala DateTimeUtilsSuite daysToMillis and millisToDays
    fails w/java 8 181-b13
  * [SPARK-24957] - Decimal arithmetic can lead to wrong values using codegen
  * [SPARK-25081] - Nested spill in ShuffleExternalSorter may access a released
    memory page
  * [SPARK-25114] - RecordBinaryComparator may return wrong result when
    subtraction between two words is divisible by
    Integer.MAX_VALUE
  * [SPARK-25144] - distinct on Dataset leads to exception due to Managed
    memory leak detected
  * [SPARK-25164] - Parquet reader builds entire list of columns once for each
    column
  * [SPARK-25402] - Null handling in BooleanSimplification
  * [SPARK-25568] - Continue to update the remaining accumulators when failing
    to update one accumulator
  * [SPARK-25591] - PySpark Accumulators with multiple PythonUDFs
  * [SPARK-25714] - Null Handling in the Optimizer rule BooleanSimplification
  * [SPARK-25726] - Flaky test: SaveIntoDataSourceCommandSuite.`simpleString is
    redacted`
  * [SPARK-25797] - Views created via 2.1 cannot be read via 2.2+
  * [SPARK-25854] - mvn helper script always exits w/1, causing mvn builds to
    fail
  * [SPARK-26233] - Incorrect decimal value with java beans and
    first/last/max... functions
  * [SPARK-26537] - update the release scripts to point to gitbox
  * [SPARK-26545] - Fix typo in EqualNullSafe's truth table comment
  * [SPARK-26553] - NameError: global name '_exception_message' is not defined
  * [SPARK-26802] - CVE-2018-11760: Apache Spark local privilege escalation
    vulnerability
  * [SPARK-26118] - Make Jetty's requestHeaderSize configurable in Spark
  * [SPARK-20715] - MapStatuses shouldn't be redundantly stored in both
    ShuffleMapStage and MapOutputTracker
  * [SPARK-25253] - Refactor pyspark connection & authentication
  * [SPARK-25576] - Fix lint failure in 2.2
  * [SPARK-24564] - Add test suite for RecordBinaryComparator
- Add _service
- Drop fix-spark-home-and-conf.patch (no longer needed since all scripts use
  find-spark-home now)
- Adjust build.sh to account for automatic Hadoop version and
  new Kafka version
- Address various packaging deficiencies (bsc#1081531):
  * Remove configuration templates from /usr/share/spark
  * Fix static versioning
  * Get rid of wildcards in %%files section
  * Improve Summary
* Sat Feb  9 2019 ashwin.agate@suse.com
- Added Restart and RestartSec to restart
  spark master and spark worker (bsc#1091479)
* Wed Mar 21 2018 ashwin.agate@suse.com
- Remove drizzle jdbc jar (bsc#1084084)
* Thu Mar  8 2018 tbechtold@suse.com
- Add fix-spark-home-and-conf.patch
  The patch fixes SPARK_HOME and SPARK_CONF_DIR in the different
  bin/spark-* scripts.
* Thu Mar  8 2018 ashwin.agate@suse.com
- Added SPARK_DAEMON_JAVA_OPTS to set java heap size
  settings in spark-worker and spark-master service
  files.
* Tue Mar  6 2018 tbechtold@suse.com
- Install /etc/spark/spark-env . This script is automatically
  read during startup and can be used for custom configuration
- Install /etc/spark/spark-defaults.conf
- Create /run/spark dir via systemd tmpfiles
- Add missing Requires/BuildRequires for systemd
- Drop openstack-suse-macros BuildRequires and use the typical
  way to create a spark user/group and homedir
- Add useful description
* Fri Feb 23 2018 dmueller@suse.com
- cleanup spec file
* Fri Feb 23 2018 jodavis@suse.com
- Fix spark-worker.service to use port 7077, avoiding conflict (bsc#1081275)
* Mon Feb 19 2018 tbechtold@suse.com
- Fix ExecStartPre bash syntax in spark-worker.service (bsc#1081275)
* Mon Jul 24 2017 jbrownell@suse.com
- Initial package
